@@ -5,21 +5,23 @@ use strict;
 
 use File::Next 0.40;
 
+use App::Ack::Plugin::Basic;
+
 =head1 NAME
 
 App::Ack - A container for functions for the ack program
 
 =head1 VERSION
 
-Version 1.86
+Version 1.88
 
 =cut
 
 our $VERSION;
 our $COPYRIGHT;
 BEGIN {
-    $VERSION = '1.86';
-    $COPYRIGHT = 'Copyright 2005-2008 Andy Lester, all rights reserved.';
+    $VERSION = '1.88';
+    $COPYRIGHT = 'Copyright 2005-2009 Andy Lester, all rights reserved.';
 }
 
 our $fh;
@@ -77,7 +79,7 @@ BEGIN {
         csharp      => [qw( cs )],
         css         => [qw( css )],
         elisp       => [qw( el )],
-        erlang      => [qw( erl )],
+        erlang      => [qw( erl hrl )],
         fortran     => [qw( f f77 f90 f95 f03 for ftn fpp )],
         haskell     => [qw( hs lhs )],
         hh          => [qw( h )],
@@ -97,7 +99,8 @@ BEGIN {
         php         => [qw( php phpt php3 php4 php5 )],
         plone       => [qw( pt cpt metadata cpy py )],
         python      => [qw( py )],
-        ruby        => [qw( rb rhtml rjs rxml erb )],
+        rake        => q{Rakefiles},
+        ruby        => [qw( rb rhtml rjs rxml erb rake )],
         scheme      => [qw( scm )],
         shell       => [qw( sh bash csh tcsh ksh zsh )],
         skipped     => q{Files, but not directories, normally skipped by ack (default: off)},
@@ -458,7 +461,11 @@ sub filetypes {
 
     return 'skipped' unless is_searchable( $filename );
 
-    return ('make',TEXT) if $filename =~ m{[$dir_sep_chars]?Makefile$}io;
+    my $basename = $filename;
+    $basename =~ s{.*[$dir_sep_chars]}{};
+
+    return ('make',TEXT)        if lc $basename eq 'makefile';
+    return ('rake','ruby',TEXT) if lc $basename eq 'rakefile';
 
     # If there's an extension, look it up
     if ( $filename =~ m{\.([^\.$dir_sep_chars]+)$}o ) {
@@ -492,7 +499,7 @@ sub filetypes {
         return;
     }
     my $header = <$fh>;
-    close_file( $fh, $filename ) or return;
+    close $fh;
 
     if ( $header =~ /^#!/ ) {
         return ($1,TEXT)       if $header =~ /\b(ruby|p(?:erl|hp|ython))\b/;
@@ -648,11 +655,11 @@ sub show_help {
     my $ignore_dirs = _listify( sort { _key($a) cmp _key($b) } keys %ignore_dirs );
 
     App::Ack::print( <<"END_OF_HELP" );
-Usage: ack [OPTION]... PATTERN [FILES]
+Usage: ack [OPTION]... PATTERN [FILE]
 
 Search for PATTERN in each source file in the tree from cwd on down.
 If [FILES] is specified, then only those files/directories are checked.
-ack may also search STDIN, but only if no FILES are specified, or if
+ack may also search STDIN, but only if no FILE are specified, or if
 one of FILES is "-".
 
 Default switches may be specified in ACK_OPTIONS environment variable or
@@ -764,6 +771,8 @@ Miscellaneous:
   --version             Display version & copyright
   --thpppt              Bill the Cat
 
+Exit status is 0 if match, 1 if no match.
+
 This is version $VERSION of ack.
 END_OF_HELP
 
@@ -903,79 +912,6 @@ sub is_interesting {
 }
 
 
-=head2 open_file( $filename )
-
-Opens the file specified by I<$filename> and returns a filehandle and
-a flag that says whether it could be binary.
-
-If there's a failure, it throws a warning and returns an empty list.
-
-=cut
-
-sub open_file {
-    my $filename = shift;
-
-    my $fh;
-    my $could_be_binary;
-
-    if ( $filename eq '-' ) {
-        $fh = *STDIN;
-        $could_be_binary = 0;
-    }
-    else {
-        if ( !open( $fh, '<', $filename ) ) {
-            App::Ack::warn( "$filename: $!" );
-            return;
-        }
-        $could_be_binary = 1;
-    }
-
-    return ($fh,$could_be_binary);
-}
-
-=head2 close_file( $fh, $filename )
-
-Close L<$fh> opened from L<$filename>.
-
-=cut
-
-sub close_file {
-    if ( close $_[0] ) {
-        return 1;
-    }
-    App::Ack::warn( "$_[1]: $!" );
-    return 0;
-}
-
-
-=head2 needs_line_scan( $fh, $regex, \%opts )
-
-Slurp up an entire file up to 100K, see if there are any matches
-in it, and if so, let us know so we can iterate over it directly.
-If it's bigger than 100K or the match is inverted, we have to do
-the line-by-line, too.
-
-=cut
-
-sub needs_line_scan {
-    my $fh = shift;
-    my $regex = shift;
-    my $opt = shift;
-
-    return 1 if $opt->{v};
-
-    my $size = -s $fh;
-
-    if ( $size > 100_000 ) {
-        return 1;
-    }
-
-    my $buffer;
-    my $rc = sysread( $fh, $buffer, $size );
-    return 0 unless $rc && ( $rc == $size );
-
-    return $buffer =~ /$regex/m;
-}
 
 # print subs added in order to make it easy for a third party
 # module (such as App::Wack) to redefine the display methods
@@ -1005,9 +941,11 @@ sub print_count0 {
 }
 
 
-=head2 search( $fh, $could_be_binary, $filename, \%opt )
+=head2 search_resource( $resource, \%opt )
 
-Main search method
+Main search method.
+
+Assumes an open resource, and that the caller will close the resource.
 
 =cut
 
@@ -1022,12 +960,12 @@ Main search method
     my $any_output;                   # has there been any output for the current file yet
     my $context_overall_output_count; # has there been any output at all
 
-sub search {
-    my $fh = shift;
-    my $could_be_binary = shift;
-    $filename = shift;
+sub search_resource {
+    my $res = shift;
     my $opt = shift;
 
+    $filename = $res->name();
+
     my $v = $opt->{v};
     my $passthru = $opt->{passthru};
     my $max = $opt->{m};
@@ -1047,7 +985,6 @@ sub search {
         $regex = qr/$opt->{regex}/;
     }
 
-
     # for context processing
     $last_output_line = -1;
     $any_output = 0;
@@ -1060,21 +997,20 @@ sub search {
     my $before_starts_at_line;
     my $after = 0; # number of lines still to print after a match
 
-    my $line;
-    while ($line = <$fh>) {
+    while ( $res->next_text ) {
         # XXX Optimize away the case when there are no more @lines to find.
         # XXX $has_lines, $passthru and $v never change.  Optimize.
         if ( $has_lines
                ? $. != $lines[0]  # $lines[0] should be a scalar
-               : $v ? $line =~ m/$regex/ : $line !~ m/$regex/ ) {
+               : $v ? m/$regex/ : !m/$regex/ ) {
             if ( $passthru ) {
-                App::Ack::print( $line );
+                App::Ack::print( $_ );
                 next;
             }
 
             if ( $keep_context ) {
                 if ( $after ) {
-                    print_match_or_context( $opt, 0, $., $line );
+                    print_match_or_context( $opt, 0, $., $_ );
                     $after--;
                 }
                 elsif ( $before_context ) {
@@ -1087,7 +1023,7 @@ sub search {
                     else {
                         $before_starts_at_line = $.;
                     }
-                    push @before, $line;
+                    push @before, $_;
                 }
                 last if $max && ( $nmatches >= $max ) && !$after;
             }
@@ -1103,12 +1039,9 @@ sub search {
 
         shift @lines if $has_lines;
 
-        if ( $could_be_binary ) {
-            if ( -B $filename ) {
-                App::Ack::print( "Binary file $filename matches\n" );
-                last;
-            }
-            $could_be_binary = 0;
+        if ( $res->is_binary ) {
+            App::Ack::print( "Binary file $filename matches\n" );
+            last;
         }
         if ( $keep_context ) {
             if ( @before ) {
@@ -1123,13 +1056,13 @@ sub search {
                 $after = $after_context;
             }
         }
-        print_match_or_context( $opt, 1, $., $line );
+        print_match_or_context( $opt, 1, $., $_ );
 
         last if $max && ( $nmatches >= $max ) && !$after;
     } # while
 
     return $nmatches;
-}   # search()
+}   # search_resource()
 
 
 =head2 print_match_or_context( $opt, $is_match, $starting_line_no, @lines )
@@ -1202,7 +1135,7 @@ sub print_match_or_context {
     return;
 } # print_match_or_context()
 
-} # scope around search() and print_match_or_context()
+} # scope around search_resource() and print_match_or_context()
 
 
 =head2 search_and_list( $fh, $filename, \%opt )
@@ -1213,8 +1146,7 @@ show lines.
 =cut
 
 sub search_and_list {
-    my $fh = shift;
-    my $filename = shift;
+    my $res = shift;
     my $opt = shift;
 
     my $nmatches = 0;
@@ -1224,7 +1156,7 @@ sub search_and_list {
     my $regex = qr/$opt->{regex}/;
 
     if ( $opt->{v} ) {
-        while (<$fh>) {
+        while ( $res->next_text ) {
             if ( /$regex/ ) {
                 return 0 unless $count;
             }
@@ -1234,7 +1166,7 @@ sub search_and_list {
         }
     }
     else {
-        while (<$fh>) {
+        while ( $res->next_text ) {
             if ( /$regex/ ) {
                 ++$nmatches;
                 last unless $count;
@@ -1243,10 +1175,10 @@ sub search_and_list {
     }
 
     if ( $nmatches ) {
-        App::Ack::print_count($filename, $nmatches, $ors, $count);
+        App::Ack::print_count( $res->name, $nmatches, $ors, $count );
     }
     elsif ( $count && !$opt->{l} ) {
-        App::Ack::print_count0($filename, $ors);
+        App::Ack::print_count0( $res->name, $ors );
     }
 
     return $nmatches ? 1 : 0;
@@ -1300,14 +1232,17 @@ sub print_files_with_matches {
 
     my $nmatches = 0;
     while ( defined ( my $filename = $iter->() ) ) {
-        my ($fh) = open_file( $filename );
-        next unless defined $fh; # error while opening file
-        $nmatches += search_and_list( $fh, $filename, $opt );
-        close_file( $fh, $filename );
-        last if $nmatches && $opt->{1};
+        my $repo = App::Ack::Repository::Basic->new( $filename );
+        my $res;
+        while ( $res = $repo->next_resource() ) {
+            $nmatches += search_and_list( $res, $opt );
+            $res->close();
+            last if $nmatches && $opt->{1};
+        }
+        $repo->close();
     }
 
-    return;
+    return $nmatches;
 }
 
 =head2 print_matches( $iter, $opt )
@@ -1325,25 +1260,37 @@ sub print_matches {
 
     my $nmatches = 0;
     while ( defined ( my $filename = $iter->() ) ) {
-        my ($fh,$could_be_binary) = open_file( $filename );
-        next unless defined $fh; # error while opening file
-        my $needs_line_scan;
-        if ( $opt->{regex} && !$opt->{passthru} ) {
-            $needs_line_scan = needs_line_scan( $fh, $opt->{regex}, $opt );
-            if ( $needs_line_scan ) {
-                seek( $fh, 0, 0 );
-            }
+        my $repo;
+        if ( $filename =~ /\.tar\.gz$/ ) {
+            App::Ack::die( 'Not working here yet' );
+            require App::Ack::Repository::Tar; # XXX Error checking
+            $repo = App::Ack::Repository::Tar->new( $filename );
         }
         else {
-            $needs_line_scan = 1;
+            $repo = App::Ack::Repository::Basic->new( $filename );
         }
-        if ( $needs_line_scan ) {
-            $nmatches += search( $fh, $could_be_binary, $filename, $opt );
+        $repo or next;
+
+        while ( my $res = $repo->next_resource() ) {
+            my $needs_line_scan;
+            if ( $opt->{regex} && !$opt->{passthru} ) {
+                $needs_line_scan = $res->needs_line_scan( $opt );
+                if ( $needs_line_scan ) {
+                    $res->reset();
+                }
+            }
+            else {
+                $needs_line_scan = 1;
+            }
+            if ( $needs_line_scan ) {
+                $nmatches += search_resource( $res, $opt );
+            }
+            $res->close();
         }
-        close_file( $fh, $filename );
         last if $nmatches && $opt->{1};
+        $repo->close();
     }
-    return;
+    return  $nmatches;
 }
 
 =head2 filetype_setup()
@@ -1504,10 +1451,20 @@ sub set_up_pager {
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2005-2008 Andy Lester, all rights reserved.
+Copyright 2005-2009 Andy Lester, all rights reserved.
 
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
+This program is free software; you can redistribute it and/or modify
+it under the terms of either:
+
+=over 4
+
+=item * the GNU General Public License as published by the Free
+Software Foundation; either version 1, or (at your option) any later
+version, or
+
+=item * the "Artistic License" which comes with Perl 5.
+
+=back
 
 =cut
 
